# -*- coding: utf-8 -*-
##############################################################################
#
#    wms_sale module for OpenERP, Module to extended sale with WMS
#    Copyright (C) 2010 SYLEAM Info Services (<http://www.Syleam.fr/>)
#              Sebastien LANGE <sebastien.lange@syleam.fr>
#
#    This file is a part of wms_sale
#
#    wms_sale is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms_sale is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
from datetime import datetime, timedelta
from tools.translate import _
from common import log_message


class StockMove(osv.osv):
    _inherit = 'stock.move'

    def _order_partner(self, cr, uid, ids, field_name, arg, context=None):
        """ Retreive sub total for raw materail of corresponding work.order
            @params field_name : not used (mandatory for openObject Framework)
            @params arg : not used
            @return (dictionnary of char) : { move_id: {'order': 'C123', 'partner': 'SYLEAM', }}
        """
        if context is None:
            context = {}
        res = {}
        for move in self.browse(cr, uid, ids, context=context):
            if move.sale_line_id and move.sale_line_id.order_id:
                res[move.id] = {'order': move.sale_line_id.order_id.name, 'partner': move.sale_line_id.order_id.partner_id.name}
            elif move.purchase_line_id and move.purchase_line_id.id:
                res[move.id] = {'order': move.purchase_line_id.order_id.name, 'partner': move.purchase_line_id.order_id.partner_id.name}
            else:
                res[move.id] = {'order': '', 'partner': ''}
        return res

    _columns = {
        'partner': fields.function(_order_partner, method=True, type='char', multi='sums', string='Partner'),
        'order': fields.function(_order_partner, method=True, type='char', multi='sums', string='Order'),
        'average_uom_id': fields.many2one('product.uom', 'Average UOM Unit', help="is the UOM of the average price"),
    }

    _defaults = {
         'average_uom_id': lambda *a: False,
    }

    def action_done(self, cr, uid, ids, context=None):
        """
        This function store average price form product or stock picking out in stock move of stock picking delivery if configure in comapny
        """
        res = super(StockMove, self).action_done(cr, uid, ids, context=context)
        company = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id

        for move in self.browse(cr, uid, ids):
            if (move.state == 'done') \
               and (move.picking_id.type == 'delivery' and company.average_price_stock_picking_delivery)\
               and (company.average_price_stock_picking_delivery_method == 'out')\
               and move.sale_line_id and move.sale_line_id.id:
                move_obj = self.pool.get('stock.move')
                move_id = move_obj.search(cr, uid, [('sale_line_id', '=', move.sale_line_id.id), ('picking_id', '!=', move.picking_id.id)], context=context)
                if move_id:
                    self.write(cr, uid, ids, {'average_price': move_obj.read(cr, uid, move_id[0], context=context)['average_price']})
        return res

    def get_picking_and_product_from_moves(self, cr, uid, ids, context=None):
        """
        the move ids must have the same picking and the same product
        """
        if context is None:
            context = {}
        picking_id = 0
        product_id = 0
        for move in self.browse(cr, uid, ids, context=context):
            if move.picking_id.id == picking_id and move.product_id.id == product_id:
                # Ok it is the same test the next product
                continue
            elif picking_id == 0 and product_id == 0:
                # it is not the same because it is the first move test
                picking_id = move.picking_id.id
                product_id = move.product_id.id
                continue
            else:
                # it is not the same.
                raise osv.except_osv(_('ERROR'), _('More move to compute without the same picking or the product'))
        return (picking_id, product_id)

    def _get_qty(self, cr, uid, id, context=None):
        """
        return the real qty in move in function of location_id and location_dest_id
        __________________________________________________________________________________________________
        |  qty  in/dest | supplier | view  | internal  | customer | inventory | procurement | production |
        --------------------------------------------------------------------------------------------------
        | supplier      | error    | error | +         | 0        | error     | 0           | error      |
        --------------------------------------------------------------------------------------------------
        | view          | error    | error | error     | error    | error     | error       | error      |
        --------------------------------------------------------------------------------------------------
        | internal      | -        | error | 0         | -        | -         | 0           | -          |
        --------------------------------------------------------------------------------------------------
        | customer      | 0        | error | +         | error    | error     | 0           | error      |
        --------------------------------------------------------------------------------------------------
        | inventory     | error    | error | +         | error    | error     | error       | error      |
        --------------------------------------------------------------------------------------------------
        | procurement   | 0        | error | 0         | 0        | error     | 0           | 0          |
        --------------------------------------------------------------------------------------------------
        | production    | error    | error | +         | error    | error     | 0           | error      |
        --------------------------------------------------------------------------------------------------
        """
        if not id:
            raise osv.except_osv(_('Move ERROR'), _('No move ID define to get quantity'))
        move = self.browse(cr, uid, id, context=context)
        loc_usage = move.location_id.usage
        loc_dest_usage = move.location_dest_id.usage
        if loc_usage == 'internal' and loc_dest_usage in ['supplier', 'customer', 'inventary', 'production']:
            return - move.product_qty
        elif loc_dest_usage == 'internal' and loc_usage in ['supplier', 'customer', 'inventary', 'production']:
            return move.product_qty
        elif loc_usage == 'procurement' and loc_dest_usage in ['supplier', 'customer', 'internal', 'production', 'procurement']:
            return 0.0
        elif loc_dest_usage == 'procurement' and loc_usage in ['supplier', 'customer', 'internal', 'production']:
            return 0.0
        elif loc_usage == 'supplier' and loc_dest_usage == 'customer':
            return 0.0
        elif loc_dest_usage == 'supplier' and loc_usage == 'customer':
            return 0.0
        elif loc_usage == 'internal' and loc_dest_usage == 'internal':
            return 0.0
        else:
            raise osv.except_osv(_('Get move quantity error'),
                                 _('the move %s(%d) with mouvement %s(%d) usage %s ==> %s(%d) usage %s is not allowed') \
                                 % (move.name, move.id, move.location_id.name, move.location_id.id, loc_usage, move.location_dest_id.name, move.location_dest_id.id, loc_dest_usage)
                                  )

    def get_previous_pump(self, cr, uid, id, date_planned, product_id, context=None):
        """
        find the move linked with a picking type in before the current move
        return the pump of the last move or the standard_price
        """
        if context is None:
            context = {}
        # make domain to find move
        product_obj = self.pool.get('product.product')
        domain = [
            ('state', '=', 'done'),  # the move must be done
            ('date_planned', '<', date_planned),  # the date planned must be after the current move
            ('product_id', '=', product_id),  # the product must be the same that the current product
            ('picking_id', '!=', False),  # the move must has got a pincking
            ('picking_id.state', '=', 'done'),  # the state of the picking must be done
            ('picking_id.type', '=', 'in'),  # compute in done on receiving
        ]
        # define the order of answer
        order_by = 'date_planned DESC, id DESC'
        # limit one record the last
        limit = 1
        move_ids = self.search(cr, uid, domain, limit=limit, order=order_by, context=context)
        pump = 0.0
        if not move_ids:
            #No receiving picking found, we take the standard_price of the product
            pump = product_obj.read(cr, uid, product_id, ['standard_price'], context=context).get('standard_price', 0.0)
        else:
            # they are only one value so take the average_price(pump) of this move
            move_rd = self.read(cr, uid, move_ids[0], ['average_price', 'average_uom_id'], context=context)
            if move_rd.get('average_uom_id', False):
                uom_id = move_rd.get('average_uom_id')[0]
                average_price = move_rd.get('average_price', 0.0)
                pump = product_obj.convert_price_in_default_unit(cr, uid, product_id, average_price, uom_id, context=context)
            else:
                pump = move_rd.get('standard_price', 0.0)
        return pump

    def get_qty_at_previous_time(self, cr, uid, product_id, date_planned, context=None):
        """
        return the qty at this date_planned
        """
        if context is None:
            context = {}
        product_obj = self.pool.get('product.product')
        # get the qantity after this moves
        to_date = datetime.strftime(datetime.strptime(date_planned, '%Y-%m-%d %H:%M:%S') + timedelta(seconds=-1), '%Y-%m-%d %H:%M:%S')
        ctx = context.copy()
        ctx['to_date'] = to_date
        return  product_obj._product_available(cr, uid, [product_id], field_names=['qty_available'], context=ctx)[product_id]['qty_available']

    def get_qty_at_time(self, cr, uid, product_id, date_planned, context=None):
        """
        return the qty at this date_planned
        """
        if context is None:
            context = {}
        product_obj = self.pool.get('product.product')
        ctx = context.copy()
        ctx['to_date'] = date_planned
        return  product_obj._product_available(cr, uid, [product_id], field_names=['qty_available'], context=ctx)[product_id]['qty_available']

    def compute_pump(self, cr, uid, ids, context=None):
        """
        compute the pump of the move and after
        """
        if context is None:
            context = {}
        outfp = context.get('log_message', False)
        product_obj = self.pool.get('product.product')
        #the move ids must have the same picking and the same product
        picking_id, product_id = self.get_picking_and_product_from_moves(cr, uid, ids, context=context)
        # get move_ids[0] data
        move_rd = self.read(cr, uid, ids[0], ['date_planned', 'price_unit'], context=context)
        # get the pump
        # already in good unit
        pump = self.get_previous_pump(cr, uid, move_rd.get('id'), move_rd.get('date_planned'), product_id, context=context)
        log_message(outfp, 'wms_sale.stock_move.compute_pump', 'The Start P.U.M.P. is %s' % repr(pump))
        # get the qantity after this moves
        stock_qty = self.get_qty_at_previous_time(cr, uid, product_id, move_rd.get('date_planned'), context=context)

        # list of move already compute
        # define the type of picking looked for
        company = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id
        picking_type = ['in']
        if company.average_price_stock_picking_internal:
            picking_type.append('internal')
        if company.average_price_stock_picking_out:
            picking_type.append('out')
        log_message(outfp, 'wms_sale.stock_move.compute_pump', 'List of picking to save PUMP %s (delivery is never here he is computed after)' % repr(picking_type))

        # make domain for search
        domain = [
            ('state', '=', 'done'),  # only the move in state done
            ('date_planned', '>=', move_rd.get('date_planned')),  # only the move after or equal the current
            ('product_id', '=', product_id),  # on this product
            ('location_id.usage', 'not in', ['view', 'procurement']),  # we dont want view
            ('location_dest_id.usage', 'not in', ['view', 'procurement']),  # we dont want view
        ]
        order_by = 'date_planned, id'

        move_ids = self.search(cr, uid, domain, order=order_by, context=context)

        delivery_ids = []
        for move_id in move_ids:
            #get new qty at the date_planned
            qty = self._get_qty(cr, uid, move_id, context=context)
            move = self.browse(cr, uid, move_id, context=context)
            log_message(outfp, 'wms_sale.stock_move.compute_pump', 'compute the move %s(%d) with picking type %s' % (move.name, move.id, move.picking_id and move.picking_id.type or 'None'))
            if move.picking_id and move.picking_id.type == 'in':
                # get the price of the move
                price_unit = product_obj.convert_price_in_default_unit(cr, uid, product_id, move.price_unit, move.product_uom and move.product_uom.id or False, context=context)
                    # the pump uom is not the same that sale uom
                #calcul new pump
                # new pump =  [ (pump * stock_qty) + ( price_unit * received qty ) ] / (stock qty + received qty )
                # only if stock_qty
                log_message(outfp, 'wms_sale.stock_move.compute_pump', 'Compute the new P.U.M.P. = ((%s * %s) + (%s * %s)) / %s'\
                            % (repr(pump), repr(stock_qty), repr(price_unit), repr(qty), repr(stock_qty + qty)))
                if stock_qty + qty > 0:
                    pump = ((pump * stock_qty) + (price_unit * qty)) / (stock_qty + qty)
                else:
                    # if stock_qty == 0 => pump = (price_unit * qty) / qty => pump = price_unit
                    # if stock_qty < 0 (example -1) => pump = ( -pump + ( price_unit * qty) ) / (qty - 1)
                      ## error cause if qty = stock_qty => division by zero
                      ##                qty < stock_qty => pump < 0 forbiden
                      ##                qty > stock_qty => pump wrong the good pump is the price_unit
                    pump = price_unit
                log_message(outfp, 'wms_sale.stock_move.compute_pump', 'Compute the new P.U.M.P. = %s' % repr(pump))
            if move.picking_id and move.picking_id.type in picking_type:
                # write the pump in average_price
                average_uom_id = move.product_id.list_price_uom_id and move.product_id.list_price_uom_id.id or move.product_uom.id
                self.write(cr, uid, move_id, {'average_price': pump, 'average_uom_id': average_uom_id}, context=context)
            elif company.average_price_stock_picking_delivery and move.picking_id.type == 'delivery':
                delivery_ids.append(move_id)
            # save new stock_qty and compute list
            stock_qty = (stock_qty + qty)

        log_message(outfp, 'wms_sale.stock_move.compute_pump', 'save the P.U.M.P. %s on product %d' % (repr(pump), product_id))
        product_obj.write(cr, uid, [product_id], {'standard_price': pump}, context=context)
        # only for the delivery
        if delivery_ids and company.average_price_stock_picking_out:
            log_message(outfp, 'wms_sale.stock_move.compute_pump', 'save the P.U.M.P. On delivery with method %s' % company.average_price_stock_picking_delivery_method)
            for move in self.browse(cr, uid, delivery_ids, context=context):
                log_message(outfp, 'wms_sale.stock_move.compute_pump', 'for move %s(%d)' % (move.name, move.id))
                # they are three mode:
                # 1 : the sale order is invoiced so we dont touch of the pump
                # 2 : the pump of the delivery is on the product (average_price_stock_picking_delivery_method == product on company) use the pump in the product
                # 3 : the pump of the delivery is on the out picking (average_price_stock_picking_delivery_method == out on company) use the pump in the move in delivery out
                if move.sale_line_id.order_id.invoiced:
                    # the sale order is invoiced
                    log_message(outfp, 'wms_sale.stock_move.compute_pump', 'Already Invoiced')
                    continue
                elif company.average_price_stock_picking_delivery_method == 'product':
                    # take the pump of the product. we have already the pump because the saved is make before and the value is in pump variable
                    self.write(cr, uid, move.id, {'average_price': pump}, context=context)
                elif company.average_price_stock_picking_delivery_method == 'out':
                    # we must search the move with the same order, the same product, this picking must be done and out type, the same name
                    domain = [
                        ('state', '=', 'done'),  # only the move in state done
                        ('product_id', '=', product_id),  # on this product
                        ('picking_id.state', '=', 'done'),  # pickinf is done
                        ('picking_id.type', '=', 'out'),  # type in picking_type must be a out
                        ('name', '=', move.name),  # the same name
                    ]
                    move_out_id = self.search(cr, uid, domain, context=context)[0]
                    move_out_average = self.read(cr, uid, move_out_id, ['average_price', 'average_uom_id', 'product_uom'], context=context)
                    average_uom_id = move_out_average.get('average_uom_id', move_out_average.get('product_uom', [False]))[0]
                    self.write(cr, uid, move.id, {'average_price': move_out_average.get('average_price', 0.0), 'average_uom_id': average_uom_id}, context=context)
                else:
                    raise osv.except_osv(_('WARNING'), _('No delivery mode found for the compute of the P.U.M.P. on delivery move'))
        return True

StockMove()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
