# -*- coding: utf-8 -*-
##############################################################################
#
#    wms_purchase module for OpenERP, Inherite purcharge for wms
#    Copyright (C) 2010 SYLEAM Info Services (<http://www.syleam.fr/>)
#              Jean-Sébastien SUZANNE <jean-sebastien.suzanne@syleam.fr>
#
#    This file is a part of wms_purchase
#
#    wms_purchase is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms_purchase is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from tools import config
from tools import ustr
import time
import netsvc


class wizard_compute_pump(osv.osv_memory):
    _name = 'wizard.compute.pump'
    _description = 'This wizard compute the pump among reception'

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False):
        """
        The view of the wizard is directly generate in function of product in each move of picking
        """
        if context is None:
            context = {}
        # we must get the translatation to display in the lang of the user
        trans_dict = {}
        if context.get('lang', False) and context.get('lang', False) != "en_US":
            cr.execute("""select src,value
                          from ir_translation
                          where lang='%s' and name= 'wizard.compute.pump'
                          and type= 'view'""" % context.get('lang'))
            for key, val in cr.fetchall():
                if not val:
                    val = key
                    trans_dict[key] = val
        # get the current picking
        picking = self.pool.get('stock.picking').browse(cr, uid, context.get('active_id'), context=context)
        # check if the wizard can be used
        if picking.type != 'in':
            raise osv.except_osv(ustr(trans_dict.get('WARNING', 'WARNING')), ustr(trans_dict.get('The picking must be a reception', 'The picking must be a reception')))
        if picking.state != 'done':
            raise osv.except_osv(ustr(trans_dict.get('WARNING', 'WARNING')), ustr(trans_dict.get('The picking must be done', 'The picking must be done')))

        # use lxml to compose the arch XML
        fields = {}
        arch = """<form string="%s"><group colspan="4">""" % ustr(trans_dict.get('Compute PUMP', 'Compute PUMP'))
        for move in picking.move_lines:
            field_name = "product_%d" % move.product_id.id
            if field_name in fields:
                continue
            arch += """<field name="%s"/>""" % field_name
            fields.update({str(field_name):
                                {
                                    'type': 'float',
                                    'string': move.product_id.name,
                                    'digit': (16, int(config['price_accuracy'])),
                                },
                           })

        arch += """     </group>
                        <group colspan="4">
                            <button colspan="2" name="cancel_pump" string="%s" type="object"/>
                            <button colspan="2" name="compute_pump" string="%s" type="object"/>
                        </group>

                    </form>""" % (ustr(trans_dict.get('Cancel', 'Cancel')), ustr(trans_dict.get('Compute', 'Compute')))
        tb = {'print': [], 'action': [], 'relate': []}
        result = {
        'arch': arch,
        'fields': fields,
        'toolbar': tb,
        }
        return result

    def default_get(self, cr, uid, fields_list, context=None):
        """
        inherit the default_get to force default value
        """
        if context is None:
            context = {}
        # get the current picking
        picking = self.pool.get('stock.picking').browse(cr, uid, context.get('active_id'), context=context)
        values = {}
        for move in picking.move_lines:
            field_name = "product_%d" % move.product_id.id
            if field_name in values:
                continue
            else:
                values[str(field_name)] = move.price_unit
        return values

    def compute_pump(self, cr, uid, ids, context=None):
        """
        make nothing
        """
        if context is None:
            context = {}
        picking_obj = self.pool.get('stock.picking')
        # get the value of the wizard
        for value in self.read(cr, uid, ids, [], context=context):
            picking_id = value.get('picking_id')
            # del picking of dict because we want only the product
            del value['picking_id']
            # del id of dict because we want only the product
            del value['id']
            #the product have only int keys
            for key in value.keys():
                if not isinstance(key, (int, long)):
                    del value[key]
            # call picking method
            picking_obj.compute_pump(cr, uid, picking_id, product_linked=value, context=context)
        # free memory they are no garbage in an osv.memory
        self.unlink(cr, uid, ids, context=context)
        return {'type': 'ir.actions.act_window_close'}

    def cancel_pump(self, cr, uid, ids, context=None):
        """
        make nothing
        """
        if context is None:
            context = {}
        # free memory they are no garbage in an osv.memory
        self.unlink(cr, uid, ids, context=context)
        return {'type': 'ir.actions.act_window_close'}

    def create(self, cr, uid, values, context=None):
        """
        it is not an inherit it is a new definition of create
        """
        if context is None:
            context = {}
        self.vaccum(cr, uid)
        self.next_id += 1
        id_new = self.next_id
        # save the order_id
        vals = {'picking_id': context.get('active_id')}
        # for each product put it in vals this { id:new price }
        for field in values:
            vals[int(field[8:])] = values[field]
        self.datas[id_new] = vals
        self.datas[id_new]['internal.date_access'] = time.time()

        self._validate(cr, uid, [id_new], context)
        wf_service = netsvc.LocalService("workflow")
        wf_service.trg_create(uid, self._name, id_new, cr)
        return id_new

    def read(self, cr, uid, ids, fields_to_read=None, context=None, load='_classic_read'):
        """
        it is not an inherit it is a new definition of read
        """
        if context is None:
            context = {}
        result = []
        #save a copy of ids to test is type
        testids = ids
        if self.datas:
            if isinstance(testids, (int, long)):
                ids = [ids]
            for id in ids:
                if id in self.datas:
                    if not fields_to_read:
                        fields_to_read = self.datas[id].keys()

                    r = {'id': id}
                    for f in fields_to_read:
                        r[f] = self.datas[id].get(f, False)
                    self.datas[id]['internal.date_access'] = time.time()
                result.append(r)
        if isinstance(testids, (int, long)):
            return result[0]
        return result

wizard_compute_pump()



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
